package org.objectweb.petals.system.launch;



import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.kernel.server.FractalHelper;
import org.objectweb.petals.kernel.server.PetalsServer;
import org.objectweb.petals.kernel.server.PetalsListener;
import org.objectweb.petals.util.JNDIUtil;

/**
 * This class is used to start and manage the Petals platform from command line
 * 
 * @author alouis, ddesjardins, Rafael Marins, rnaudin
 * @version $Rev: 461 $ $Date: 2006-05-24 14:12:45 +0200 (mer., 24 mai 2006) $
 */
public class StandaloneLauncher implements PetalsListener {

	public static final String START_COMMAND = "start";

	public static final String STOP_COMMAND = "stop";

	public static final String SHUTDOWN_COMMAND = "shutdown";

	public static final String VERSION_COMMAND = "version";

	public static final String JNDIBROWSE_COMMAND = "JNDIBrowser";


	public StandaloneLauncher() {
		super();
	}

	/**
	 * The launcher instance to process the main program
	 */
	private static StandaloneLauncher launcher;

	/**
	 * The Petals server instance
	 */
	private static PetalsServer petalsServer;

	/**
	 * System exit hook used if the command line is used
	 */
	private static SystemExitHook systemExitHook;

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		// Process the Command and exits abnormaly if necessary
		try {
			launcher = new StandaloneLauncher();

			petalsServer = new PetalsServer();

			String command = (args.length == 0 || args[0].startsWith("-")) ? START_COMMAND : args[0];


			if (STOP_COMMAND.equals(command)) {
				System.out.println("Petals is stopping...");
				petalsServer.triggerStop(false);
				System.out.println("Petals is stopped");
			} else if (SHUTDOWN_COMMAND.equals(command)) {
				System.out.println("Petals is stopping...");
				petalsServer.triggerStop(true);
				System.out.println("Petals is stopped");
			} else if (VERSION_COMMAND.equals(command)) {
				petalsServer.getServerVersion();
			} else if (JNDIBROWSE_COMMAND.equals(command)) {
				System.out.println(JNDIUtil.browseJNDI(args));
			} else if (START_COMMAND.equals(command)) {

				// show the fractal explorer mode if asked
				if (args.length > 0 && "-explore".equalsIgnoreCase(args[0])) {
					petalsServer.setObserver(true);
				}
				petalsServer.addStopListener(launcher);


				// add a hook if a terminate signal is sent from the command line
				systemExitHook = new SystemExitHook(petalsServer);
				Runtime.getRuntime().addShutdownHook(systemExitHook);

				System.out.println("Petals is starting...");
				petalsServer.start();
				System.out.println("##### Petals container correctly started #####");


				// show the commandLine mode if asked
				if (args.length > 0 && "-console".equalsIgnoreCase(args[0])) {
					launcher.commandLineMode();
				}
			}
		} catch (Throwable e) {
			e.printStackTrace(System.err);
			System.exit(-1);
		}
	}

	/**
	 * Show a command line to interact with Petals TODO we pass the PetalsAdmin
	 * object to the CommandReader to perform some actions. Find a better way.
	 * 
	 * @throws NoSuchInterfaceException
	 * @throws ADLException 
	 * @throws PetalsException 
	 */
	protected void commandLineMode() throws NoSuchInterfaceException, ADLException, PetalsException {
		Component systemState = FractalHelper.getFractalHelper().getComponentByName(
				petalsServer.getPetalsContentController(), "PetalsAdminService");

		PetalsAdminServiceMBean petalsService = (PetalsAdminServiceMBean) systemState
		.getFcInterface("service");

		CommandReader console = new CommandReader(petalsService);

		console.read();
	}

	public void onPetalsStarted() {

	}

	public void onPetalsStopped() {
		// remove the System Hook to avoid the Petals server stop() called a second time
		Runtime.getRuntime().removeShutdownHook(systemExitHook);
		System.out.println("Petals is stopped");

		System.exit(0);
	}
}
